# coding: UTF-8

require 'rbconfig'
require 'terrapin/os_detector'
require 'terrapin/command_line'
require 'terrapin/command_line/output'
require 'terrapin/command_line/multi_pipe'
require 'terrapin/command_line/runners'
require 'terrapin/exceptions'

module Terrapin
end
