# coding: UTF-8

require 'terrapin/command_line/runners/backticks_runner'
require 'terrapin/command_line/runners/process_runner'
require 'terrapin/command_line/runners/posix_runner'
require 'terrapin/command_line/runners/popen_runner'
require 'terrapin/command_line/runners/fake_runner'
